#************************************************************ 
#
# Component       CNF  
#
# Synopsis        Network and CGTP interfaces
#
#
# Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)CGTP.pm 1.7 02/03/15 SMI"
#
#************************************************************

#=head1 Network and CGTP interfaces
#
#=over 4
#
#=cut

package CGTP ;

use LOG ;
use CMM ;

#
#
#
my $G_category = "CGTP" ;

#
#  network devices related information
#
my @G_net_device_array ;     # array of network device names
my $G_net_device_count = 0 ; # # of network devices
my %G_net_device_type ;      # !
my %G_net_device_flags ;     # !
my %G_net_device_mtu ;       # !
my %G_net_device_index ;     # !
my %G_net_device_inet ;      # ! hashes indexed by network device name
my %G_net_device_ipaddr ;    # ! 
my %G_net_device_netmask ;   # ! 
my %G_net_device_broadcast ; # !
my %G_net_device_ether ;     # !
my %G_net_device_hostname ;  # !
my $G_physical_net_device_count = 0 ; # # of physical network devices

#
#  netstat related information
#
my @G_netstat ;
my $G_netstat_count ;

#
#  updated when loading information
#
my $G_physical_up_and_running_count = 0 ;
my $G_MAC_list ;                     # list of all different MAC addresses
my $G_MAC_count = 0 ;                # # of different MAC addresses
my $G_MAC_twice = FALSE ;            # true if same address found twice

#
#  configuration data
#

my %G_net_interface_number;
my $G_net_physical_interfaces ;
my $G_net_devices_for_master ;
my $G_net_devices_for_others ;
my $G_net_devices_to_test ;

my $G_load_done = FALSE ;
my $G_load_hostnames_done = FALSE ;

my $G_cgtp_interface = "cgtp0" ;

#--------------------------------------------------------------
#
#  build_IP_address
#
#      1 : interface
#
#--------------------------------------------------------------

sub build_IP_address
  {
    my ($L_node_id, $L_interface) = @_ ;

    $address="$MAIN::E_subnet0.$CMM::E_variables{CMM_DOMAIN_ID}.$G_net_interface_number{$L_interface}.$L_node_id" ;

    return $address ;
  }

#--------------------------------------------------------------
#
#  prepare
#
#      look for some information regarding network devices
#      and set some indicators
#
#--------------------------------------------------------------

sub prepare
  {
    my $L_up ;
    my $L_running ;

    #
    #  count and flag all ethernet devices up and running
    #  check at the same time that the same MAC address is not used twice
    #  get the status of CGTP device
    #

    LOG::print_msg(2, "    -> analysis of data") ;
    foreach $L_device (@G_net_device_array)
      {
	$L_up = FALSE;
	$L_running = FALSE;
	foreach $i (split(',',$G_net_device_flags{$L_device}))
	  {
	    if ($i eq UP )
	      {
		$L_up = TRUE ;
		LOG::print_msg(2, "    -> $L_device is UP") ;
	      }
	    if ($i eq RUNNING )
	      {
		$L_running = TRUE ;
		LOG::print_msg(2, "    -> $L_device is RUNNING") ;
	      }
	  }

	if ( ($L_up eq TRUE) && ($L_running eq TRUE) )
	  {
	    if (($L_device !~ /^cgtp/) && ($G_net_device_type{$L_device} eq PHYSICAL))
	      {
		#
		#  physical devices
		#

		$G_physical_up_and_running_count += 1;
		LOG::print_msg(2, "    -> $L_device is a physical device UP and RUNNING") ;

		#
		# add it in the list of used MAC addresses
		# (if we are root)
		if ($MAIN::E_isRoot eq TRUE)
		  {
#		    LOG::print_msg(2, "    -> checking if $G_net_device_ether{$L_device} already used") ;
		    for ($i = 0, $found = FALSE ;
			 ($i < $G_MAC_count) && ($found ne TRUE) ;
			 $i++)
		      {
			LOG::print_msg(2, "        compare with $G_MAC_list[$i]") ;
			if ($G_MAC_list[$i] eq $G_net_device_ether{$L_device})
			  {
			    LOG::print_msg(2, "        -> found twice") ;
			    $found = TRUE ;
			    $G_MAC_twice = TRUE ;
			  }
		      }
		    if ($found eq FALSE)
		      {
			$G_MAC_list[$G_MAC_count] =
			  $G_net_device_ether{$L_device} ;
			$G_MAC_count += 1 ;
		      }
		  }
	      }
	  }
      }
  }

#--------------------------------------------------------------
#
#  build_interface_lists
#
#
#--------------------------------------------------------------

sub build_interface_lists
  {
    
    LOG::print_msg(1, "CMM_USE_CGTP: $CMM::E_variables{CMM_USE_CGTP}") ;

    if ($CMM::E_variables{CMM_USE_CGTP} ne "NO") {

      $G_net_physical_interfaces = "$CMM::E_variables{CMM_NIC0} $CMM::E_variables{CMM_NIC1}" ;
      $G_net_devices_to_test = "$G_cgtp_interface $CMM::E_variables{CMM_NIC0} $CMM::E_variables{CMM_NIC1}" ;
      $G_net_devices_for_master = "$G_cgtp_interface $G_cgtp_interface:1 $CMM::E_variables{CMM_NIC0} $CMM::E_variables{CMM_NIC0}:1 $CMM::E_variables{CMM_NIC1} $CMM::E_variables{CMM_NIC1}:1" ;
      $G_net_devices_for_others = "$G_cgtp_interface $CMM::E_variables{CMM_NIC0} $CMM::E_variables{CMM_NIC1}" ;
      %G_net_interface_number = ( "$G_cgtp_interface" => 3,
				  "$CMM::E_variables{CMM_NIC0}" => 1,
				  "$CMM::E_variables{CMM_NIC1}" => 2
				) ;

    } else {

      $G_net_physical_interfaces = "$CMM::E_variables{CMM_NIC0}" ;
      $G_net_devices_to_test = "$CMM::E_variables{CMM_NIC0}" ;
      $G_net_devices_for_master = "$CMM::E_variables{CMM_NIC0} $CMM::E_variables{CMM_NIC0}:1" ;
      $G_net_devices_for_others = "$CMM::E_variables{CMM_NIC0}" ;
      %G_net_interface_number = ( "$CMM::E_variables{CMM_NIC0}" => 1) ;

    }


    LOG::print_msg(1, "physical device to test:   $G_net_devices_to_test") ;
    LOG::print_msg(1, "interface for master:      $G_net_devices_for_master") ;
    LOG::print_msg(1, "interface for other nodes: $G_net_devices_for_others") ;
  }

#--------------------------------------------------------------
#
#  load
#
#      load information regarding network devices and cgtp
#
#--------------------------------------------------------------

sub load
  {
    if ($G_load_done eq TRUE)
      {
	return ;
      }

    $G_load_done = TRUE ;

    #
    # get the complete list of network device
    #

    LOG::print_msg(2, "    execution '/sbin/ifconfig -a") ;
    @L_result = qx'/sbin/ifconfig -a';

    foreach $L_line (@L_result)
      {
	chomp($L_line) ;
	LOG::print_msg(2,"    analyzing $L_line") ;
	#
	# check if the line contains an ethernet device reference
	#
	if (($L_line =~ m/^([a-z0-9]*): /o) ||
	    ($L_line =~ m/^([a-z0-9]*:[0-9]+): /o))
	  {

	    #
	    # keep only the ethernet device reference
	    # remove alias and loopback
	    #
	    $L_device = $1;
	    LOG::print_msg(2, "    --> device: $L_device") ;
	    if ($L_device ne "lo0")
		{
		  if (MAIN::isInArray(\@G_net_device_array,
				$G_net_device_count,$L_device) eq FALSE)
		    {
		      $G_net_device_array[$G_net_device_count] = $L_device ;
		      if ($L_line =~ m/^([a-z0-9]*): /o)
			{
			  $G_net_device_type{$L_device} = PHYSICAL ;
			  $G_physical_net_device_count++ ;
			  LOG::print_msg(2, "    --> $L_device is a physical device") ;
			}
		      else
			{
			  $G_net_device_type{$L_device} = ALIAS ;
			  LOG::print_msg(2, "    --> $L_device is an alias") ;
			}
		      $G_net_device_count++;
		    }
		}
	  }
      }

    #
    #  for each network interface, get the information
    # that can be retrieved by the 'ifconfig -a' command
    #
    foreach $L_device (@G_net_device_array)
      {
	@L_result = qx{/sbin/ifconfig $L_device};
	foreach $L_line (@L_result)
	  {
	    if ($L_line =~ /flags=[0-9]*<([^>]*)>/o)
	      {
	        $G_net_device_flags{$L_device} = $1 ;
	      }
	    if ($L_line =~ /mtu ([0-9]*)/o)
	      {
		$G_net_device_mtu{$L_device} = $1;
	      }
	    if ($L_line =~ /index ([0-9]*)/o)
	      {
		$G_net_device_index{$L_device} = $1;
	      }
	    if ($L_line =~ /inet ([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/o)
	      {
		$G_net_device_ipaddr{$L_device} = $1;
	      }
	    if ($L_line =~ /netmask ([0-9a-f]*)/o)
	      {
		$G_net_device_netmask{$L_device} = $1;
	      }
	    if ($L_line =~ /broadcast ([0-9]*\.[0-9]*\.[0-9]*\.[0-9]*)/o)
	      {
		$G_net_device_broadcast{$L_device} = $1;
	      }
	    if ($L_line =~ /ether ([0-9aa-f:]*)/o)
	      {
		$G_net_device_ether{$L_device} = $1;
	      }
	  }
      }

    prepare ;
  }

#--------------------------------------------------------------
#
#  load_hostnames
#
#      load hostname by reading content of /etc/hostname.<nic>
#
#--------------------------------------------------------------

sub load_hostnames
  {
    if ($G_load_hostnames_done eq TRUE)
      {
	return ;
      }
    $G_load_hostnames_done = TRUE ;

    foreach $L_device (@G_net_device_array)
      {
	if ($G_net_device_type{$L_device} eq PHYSICAL)
	  {
	    LOG::print_msg(1,"    --> access to /etc/hostname.$L_device") ;
	    if (-e "/etc/hostname.$L_device")
	      {
		open(FH, "/etc/hostname.$L_device") ;
		chomp($line = <FH>) ;
		close(FH) ;
		$G_net_device_hostname{$L_device} = $line ;
		LOG::print_msg(1,"    --> /etc/hostname.$L_device contains $G_net_device_hostname{$L_device}") ;
	      }
	  }
      }

    $G_net_device_hostname{$G_cgtp_interface} = $MAIN::E_node_name ;
    
  }

#--------------------------------------------------------------
#
#  exist_device
#
#      return TRUE if the device specified as an argument
#      exists in the device list
#
#--------------------------------------------------------------

sub exist_device
  {
    my ($L_device) = @_ ;

    foreach $device (@G_net_device_array)
      {
	if ($device eq $L_device)
	  {
	    return TRUE ;
	  }
      }
    return FALSE ;
  }

#--------------------------------------------------------------
#
#  ready
#
#      return TRUE if the device specified as an argument
#      is up and running
#      assumption: existence has been previously tested
#
#--------------------------------------------------------------

sub ready
  {
    my ($L_device) = @_ ;

    $L_up = FALSE;
    $L_running = FALSE;
    foreach $i (split(',',$G_net_device_flags{$L_device}))
      {
	if ($i eq UP )
	  {
	    $L_up = TRUE ;
	  }
	if ($i eq RUNNING )
	  {
	    $L_running = TRUE ;
	  }
      }

    if ( ($L_up eq TRUE) && ($L_running eq TRUE) )
      {
	return TRUE ;
      }
    return FALSE ;
  }

#--------------------------------------------------------------
#
#  load_netstat_info
#
#      load netstat information in order to allow further analysis
#
#--------------------------------------------------------------

sub load_netstat_info
  {
    @L_result = qx{netstat -rn} ;
    $G_netstat_count = 0 ;

    foreach $line (@L_result)
      {
	chomp($line) ;
	if ($line =~ m/([0-9\.]+)[\s\t]+([0-9\.]+)[\s\t]+([A-Z]*)[\s\t]+([0-9]+)[\s\t]+([0-9]+)[\s\t]+([0-9a-z:]*)/)
	  {
	    LOG::print_msg(2,"destination=$1 gateway=$2 flags=$3 ref=$4 use=$5 interface=$6\n") ;
	    $G_netstat{destination}[$G_netstat_count] = $1 ;
	    $G_netstat{gateway}[$G_netstat_count]     = $2 ;
	    $G_netstat{flags}[$G_netstat_count]       = $3 ;
	    $G_netstat{ref}[$G_netstat_count]         = $4 ;
	    $G_netstat{use}[$G_netstat_count]         = $5 ;
	    $G_netstat{interface}[$G_netstat_count]   = $6 ;
	    $G_netstat_count++ ;
	  }
      }
  }

#--------------------------------------------------------------
#
# search_route
#
#--------------------------------------------------------------

sub search_route
  {
    ($destination, $gateway) = @_ ;

    for (my $i = 0 ; $i < $G_netstat_count ; $i++)
      {
	if (($G_netstat{destination}[$i] eq $destination) && ($G_netstat{gateway}[$i] eq $gateway))
	  {
	    return TRUE ;
	  }
      }
    return FALSE ;
  }


#--------------------------------------------------------------
#
#  check_route
#
#--------------------------------------------------------------

sub check_route
  {

    my $L_status = OK ;
    my $interface ;
    my $result ;
    my $gateway ;
    my $cgtpip ;
    my $interfaceip ;
    my $i ;
    my $node_id ;
    my $node_name ;

    # check that there is a cgtp address of each node is routed to
    # each address of physical interface of this node

    for ($i = 0 ; $i < $CMM::E_cluster_node_count ; $i++)
      {
	if ($i != $CMM::E_cluster_local_node)
	  {
	    foreach $interface (split(" ",$G_net_physical_interfaces))
	      {
		$cgtpip = build_IP_address($CMM::E_cluster_node{node_id}[$i], $G_cgtp_interface) ;
		$interfaceip = build_IP_address($CMM::E_cluster_node{node_id}[$i], $interface) ;
		$result = search_route($cgtpip, $interfaceip) ;
		if ($result ne TRUE)
		  {

#=item Destination <CGTP ip>, gateway <interface IP>: no route defined
#
#    To be routed, CGTP protocol need a route to be defined for each interface.
#
#=cut
			
		    LOG::print_error("Destination $cgtpip, gateway $interfaceip: no route defined") ;
		    $L_status = FAILED ;
		  }
		LOG::print_result("CGTP route for $CMM::E_cluster_node{node_name}[$i] through $interface", $L_status) ;
	      }
	  }
      }

    # check that there is a route also for the master and vice-master address
    foreach $node_id ($MAIN::E_master_node_id,$MAIN::E_vicemaster_node_id)
      {
	foreach $interface (split(" ",$G_net_physical_interfaces))
	  {
	    $cgtpip = build_IP_address($node_id, $G_cgtp_interface) ;
	    $interfaceip = build_IP_address($node_id, $interface) ;
	    $result = search_route($cgtpip, $interfaceip) ;
	    if ($result ne TRUE)
	      {
		LOG::print_error("Destination $cgtpip, gateway $interfaceip: no route defined") ;
		$L_status = FAILED ;
	      }
	    $nodename = "master" ;
	    if ($node_id == $MAIN::E_vicemaster_node_id)
	      {
		$nodename = "vice-master" ;
	      }
	    LOG::print_result("CGTP route for $nodename through $interface", $L_status) ;
	  }
      }
  }

#--------------------------------------------------------------
#
#  check_installation
#
#      check correct configuration of network devices
#      - at least 2 up and running for a real high-availability
#      - not the same MAC address twice
#
#--------------------------------------------------------------

sub check_installation
  {
    my $L_status ;
    my $L_test ;

    #
    #  check that at least 2 ethernet devices are present
    #

    LOG::print_msg(0,"") ;
    LOG::print_msg(0,"Network devices availability") ;
    LOG::print_msg(0,"") ;

    load ;

    $L_test = "at least 2 Ethernet devices present" ;
    $L_status = OK ;
    if ($G_physical_net_device_count < 2)
      {
	$L_status = FAILED_SKIP ;
	LOG::print_result($L_test,$L_status) ;
	return ;
      }
    LOG::print_result($L_test,$L_status) ;
    
    #
    # if test is ok, check at at least two of these ethernet devices
    # are up and running by checking the flags
    # check also that, for the running devices,  MAC addresses are different
    # in order to allow CGTP to be used (only if we are root)
    #

    $L_test = "at least 2 Ethernet devices up and running" ;
    $L_status = OK ;
    
    if ($G_physical_up_and_running_count < 2)
      {
	$L_status = FAILED ;
      }
    LOG::print_result($L_test,$L_status) ;
    
    $L_test = "not the same MAC address used twice" ;
    $L_status = OK ;
    if ($MAIN::E_isRoot eq TRUE)
      {
	if ($G_MAC_twice eq TRUE)
	  {
	    $L_status = FAILED ;
	  }
      }
    else
      {
	$L_status = "must be root";
      }
    LOG::print_result($L_test,$L_status) ;
  }

#--------------------------------------------------------------
#
#  check_configuration
#
#      check network information to eliminate potential problem
#
#--------------------------------------------------------------

sub check_configuration
  {
    my $L_status = OK ;

    LOG::print_msg(0, "") ;
    LOG::print_msg(0, "CGTP and network configuration") ;
    LOG::print_msg(0, "") ;

    load ;
    load_hostnames ;
    build_interface_lists ;

    #
    # test the existence of /etc/hostname.<NICx>
    #
    $L_status = OK ;
    foreach $L_interface (split(" ",$G_net_physical_interfaces))
      {
	LOG::print_msg(1,"    test existence of /etc/hostname.$L_interface");
	if (! -e "/etc/hostname.$L_interface")
	  {

#=item /etc/hostname.<interface> does not exists
#
#    This file is required and should contain the name of the node
#    for the specified interface.
#
#=cut
	    LOG::print_error("/etc/hostname.$L_interface does not exist");
	    $L_status = FAILED ;
	  }
      }
    LOG::print_result("/etc/hostname.* must exist for each interface",
		      $L_status) ;
    
    #
    #
    # hostname and IP address of each interface must be defined in /etc/hosts
    #
    #  (host names have been update by "load" function)
    #
    $L_status = OK ;
    foreach $L_interface (split(" ",$G_net_devices_to_test))
      {
        if (exist_device($L_interface) eq FALSE)
          {
            next ;
          }
	$hostname = $G_net_device_hostname{$L_interface} ;

	if ($hostname eq "")
	  {

#=item host name not defined for <interface>
#
#    Either /etc/hostname does not exist or does not contain a name.
#
#=cut

	    LOG::print_error("host name not defined for $L_interface");
	    $L_status = FAILED ;
	    next ;
	  }
	$ip = build_IP_address($CMM::E_cluster_node{node_id}[$CMM::E_cluster_local_node], $L_interface) ;
	$found = FALSE ;
	open(FH, "</etc/hosts") ;
	while (<FH>)
	  {
	    chomp($line = $_) ;
	    $line =~ s/#.*// ;
	    #
	    # <IP address> <hostname> ...
	    #
	    if ($line =~ m/[\s\t]*([0-9.]*)[\s\t]*(.*)/)
              {
		if ($1 eq $ip)
		  {
		    LOG::print_msg(3, "definition IP: $1 hostname: $2 being processed") ;
		    if ($found eq TRUE)
		      {

#=item <IP address> defined twice in /etc/hosts
#
#    Please check entries to be sure that only one hostname is defined
#    for the specified IP address.
#
#=cut
			
			LOG::print_error("$ip defined twice in /etc/hosts") ;
			$L_status = FAILED ;
		      }
		    else
		      {
			$found = TRUE ;

			# test each host names for the given IP address
			my $one_found = FALSE ;
			foreach $current_name (split(" ",$2)) {
			  if ($current_name eq $hostname) {
			    $one_found = TRUE ;
			    last ;
			  }
			}
			if ($one_found ne TRUE)
			  {

#=item invalid host name for <interface> (<IP address>): (<current> instead of <expected>)
#
#    The host name specified for <interface> does not match the name found
#    in /etc/hostname.<interface> for a physical interface or in the
#    cluster nodes table for the cgtp interface.
#
#=cut

			    LOG::print_error("invalid host name for $L_interface ($ip): ($2 instead of $hostname)") ;
			    $L_status = FAILED ;
			  }
		      }
		  }
	      }
	  }
	if ($found eq FALSE)
	  {

#=item <interface> (<IP address>) has no associated host name in /etc/hosts
#
#    The <IP address> corresponding to the <interface> is not defined
#    in /etc/hosts.
#
#=cut
    
	    LOG::print_error("$L_interface ($ip) has no associated host name in /etc/hosts") ;
	    $L_status = FAILED ;
	  }
	close(FH) ;
      }
    LOG::print_result("node name for each interface defined in /etc/hosts", $L_status) ;
  }

#--------------------------------------------------------------
#
#  check_starting
#
#      check when CGTP has been started
#
#--------------------------------------------------------------

sub check_starting
  {
    my $L_status ;
    my $L_test ;
    my $L_list ;
    my $i ;
    my $expected ;

    LOG::print_msg(0, "") ;
    LOG::print_msg(0, "CGTP availability") ;
    LOG::print_msg(0, "") ;

    load ;
    load_hostnames ;
    build_interface_lists ;

    if (CMM::is_master_eligible eq TRUE)
      {
	$L_list = $G_net_devices_for_master ;
      }
    else
      {
	$L_list = $G_net_devices_for_others ;
      }

    LOG::print_msg(2, "    list of devices to check: $L_list") ;

    foreach $L_device (split(" ",$L_list))
      {
	$L_test = "$L_device device exists" ;
	$L_status = OK ;
	if (exist_device($L_device) ne TRUE)
	  {
	    $L_status = FAILED ;
	  }
	LOG::print_result($L_test,$L_status) ;
    
	if ($L_status eq OK)
	  {
	    $L_test = "$L_device device up and running" ;
	    $L_status = FAILED ;
	    if (ready("$L_device") eq TRUE)
	      {
		$L_status = OK ;
	      }
	    LOG::print_result($L_test,$L_status) ;
	  }
      }

    #
    #  check the IP address
    #
    #  IP address must look like subnet0.{domain_id}.{interface}.{node_id}
    #
    $L_status = OK ;
    foreach $L_interface (split(" ",$G_net_devices_to_test))
      { 
	if (exist_device($L_interface) eq TRUE)
	  {
	    $expected = build_IP_address($CMM::E_cluster_node{node_id}[$CMM::E_cluster_local_node],
$L_interface) ;

	    LOG::print_msg(2, "    test IP address of $L_interface: expected:$expected actual=$G_net_device_ipaddr{$L_interface}\n") ;
	    if ($expected ne $G_net_device_ipaddr{$L_interface})
	      {

#=item <interface>: invalid IP address (<current> instead of <expected>)
#
#    The IP address configured for the specified interface is not correct.
#
#=cut

		$L_status = FAILED ;
		LOG::print_error("$L_interface: invalid IP address ($G_net_device_ipaddr{$L_interface} instead of $expected)") ;
	      }
	  }
      }
    LOG::print_result("Node Id and IP address coherency", $L_status) ;

    #
    #  check the netmask
    #
    $L_test = "Check netmasks (ffffff00)" ;
    $L_status = OK ;
    foreach $L_device (@G_net_device_array)
      {
	$L_test = "check $L_device netmask (ffffff00)" ;
	$L_status = OK ;
	if ($G_net_device_netmask{$L_device} ne "ffffff00")
	  {

#=item <interface> netmask: <current> instead of <expected>
#
#    The netmask defined for <interface> is not correct.
#    Please check the /etc/netmask file.
#
#=cut 
	    LOG::print_error("$L_device netmask: $G_net_device_netmask{$L_device} instead of ffffff00") ;
	    $L_status = FAILED ;
	  }
	LOG::print_result($L_test,$L_status) ;
      }

    #
    # check accessibility
    #
    LOG::print_msg(0,"Following tests perform network access through available interfaces");
    LOG::print_msg(0,"Can take a long time ...") ;

    #
    #  perform network access through cgtp, eth0 and eth1
    #

    $L_list = $G_net_devices_to_test ;
    for ($i = 0 ; $i < $CMM::E_cluster_node_count ; $i++)
      {
	if ($i ne $CMM::E_cluster_local_node)
	  {
	    LOG::print_msg(1,"check accessibility of node $CMM::E_cluster_node{node_name}[$i]\n") ;
	    foreach $L_device (split(" ", $G_net_devices_to_test))
	      {
                if (exist_device($L_device) eq FALSE)
                  {
                  next ;
                  }
		if (ready($L_device) eq FALSE)
		  {
		    LOG::print_msg(2,"    $L_device not ready: not tested");
		    next ;
		  }
		LOG::print_msg(2,"    $L_device ready: to be tested");
		$L_status = OK ;
		$L_test = "$CMM::E_cluster_node{node_name}[$i] reachable through $L_device" ;
		#
		#  build the address
		#
		$address=build_IP_address($CMM::E_cluster_node{node_id}[$i], $L_device) ;

		LOG::print_msg(1,"    ping node $CMM::E_cluster_node{node_name}[$i] ($address)") ;
		$command = "/usr/sbin/ping -i $L_device $address 3" ;
		LOG::print_msg(2, "  execute: $command") ;
		$L_result = qx{$command} ;
		LOG::print_msg(2, "      $L_result") ;
		if ($L_result =~ m/no answer/)
		  {
		    $L_status = FAILED ;
		  }
		LOG::print_result($L_test,$L_status) ;
	      }
	  }
      }

    if ($CMM::E_variables{CMM_USE_CGTP} ne "NO") {
      LOG::print_msg(0,"") ;
      LOG::print_msg(0,"Routing") ;
      LOG::print_msg(0,"") ;

      # check route
      if (CMM::is_master_eligible eq TRUE)
	{
	  $L_status = OK ;
	  if (! -f "/etc/notrouter") 
	    {
	      $L_status = FAILED ;
	    }
	  LOG::print_result("Routing mechanism disabled (/etc/notrouter exists)", $L_status) ;
	}

      load_netstat_info ;
      check_route ;
    }

  }
#--------------------------------------------------------------
#
#  display
#
#--------------------------------------------------------------

sub display
  {
    print "\n" ;
    print "Network interfaces\n" ;
    print "\n" ;

    load ;
    load_hostnames ;
    build_interface_lists ;

    if (CMM::is_master_eligible eq TRUE)
      {
	$L_list = $G_net_devices_for_master ;
      }
    else
      {
	$L_list = $G_net_devices_for_others ;
      }

    foreach $L_interface (split(" ",$L_list))
      {
	print "    Interface:             $L_interface\n" ;
	if (exist_device($L_interface) eq FALSE)
	    {
	      print "    -> not available\n\n" ;
	      next ;
	    }
	print "    IP address:            $G_net_device_ipaddr{$L_interface}\n" ;
	print "    Broadcast address:     $G_net_device_broadcast{$L_interface}\n" ;
	print "    Flags:                 $G_net_device_flags{$L_interface}\n" ;
	print "    Netmask:               $G_net_device_netmask{$L_interface}\n" ;

	if ($G_net_device_type{$L_interface} ne PHYSICAL)
	  {
	    print "\n" ;
	    next ;
	  }
	print "    MAC address:           $G_net_device_ether{$L_interface}\n" ;
	print "    Associated host name:  $G_net_device_hostname{$L_interface}\n" ;
	print "\n" ;
      }
  }
#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

#=back
#
#=cut

{
}
