#************************************************************ 
#
# Component       CNF  
#
# Synopsis        Cluster Reliable File Server
#
#
# Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)CRFS.pm 1.14 02/04/03 SMI"
#
#************************************************************

#=head1 Cluster Reliable File Server
#
#=over 4
#
#=cut

package CRFS ;

$E_crfs_configuration_file = "/etc/opt/SUNWcgha/crfs.conf" ;
$E_nhcrfs_configuration_file = "/etc/opt/SUNWcgha/nhcrfs.conf" ;
$E_sd_configuration_file = "/etc/opt/SUNWscm/sd.cf" ;


my $G_config_file_loaded = FALSE ;
my %G_slice ;
my $G_slice_count = 0 ;
my %G_nhcrfsconf = (
		    "CRFS_OPTIONS" => "" ,
		    "CRFS_NICCGTP" => "" ,
		    "CRFS_NIC0"    => "" ,
		    "CRFS_NIC1"    => "" ,
		    "CRFS_USE_CGTP" => "",
		   ) ;

my $G_rdcadm = "/usr/opt/SUNWesm/sbin/rdcadm" ;

my @G_partition ;
my $G_partition_count = 0 ;

my $G_mb = 1024 * 1024 ;
my $G_sector = 0 ;
my $G_svm_sector = 0 ;
my %G_mountpoint = ( "/"                 => { "size" => 1500 * $G_mb} ,
		     "/export"           => { "size" => 2000 * $G_mb} ,
		     "/SUNWcgha/local"   => { "size" => 1512 * $G_mb} ,
		     ) ;

my $G_use_svm=FALSE;
my $G_root_slice;

#--------------------------------------------------------------
#
# integer division (if there is a remaining, take the following
# integer value).
#		    
#--------------------------------------------------------------

sub divrounded
  {
    my ($value, $mod) = @_ ;

    my $res = int(($value-1)  / $mod + 1) ;
    return $res;
  }
#--------------------------------------------------------------
#
# get root slice
#
#--------------------------------------------------------------

sub load_root_slice {

    #
    # determine slice for root
    #
    chomp($result = qx{/usr/sbin/mount | grep "^\/ on"}) ;
    LOG::print_msg(3, "mount | grep: $result") ;

    if ($result !~ m/\/[ \t]*on[ \t]*([0-9a-z\/]*)/)
      {

#=item can not determine root file system
#
#    No root partition found when executing "mount" command.
#    Please correct the problem.
#
#=cut

	LOG::print_error("can not determine root file system") ;
	return FALSE ;
      }

    $G_root_slice=$1;
}

#--------------------------------------------------------------
#
#  load_config_file
#
#    load the CRFS config file
#
#--------------------------------------------------------------

sub load_config_file
  {
    my $command ;

    if ($G_config_file_loaded eq TRUE)
      {
	return TRUE ;
      }

    if (OS::check_file($E_crfs_configuration_file) ne TRUE)
      {
	return FALSE ;
      }
    if (OS::check_file($E_nhcrfs_configuration_file) ne TRUE)
      {
	return FALSE ;
      }

    open(FILE, "<$E_crfs_configuration_file") ;

    while (<FILE>)
      {
	$L_line = $_ ;
	$L_line =~ s/#.*// ;
	if ($L_line =~ m/[\s\t]*([^\s\t]*)[\s\t]*=[\s\t]*([^\s\t]*)[\s\t]*/)
	  {
	    $command = $1 ;
	    if ($command eq "slice")
	      {
		$L_line =~ m/[\s\t]*([^\s\t]*)[\s\t]*=[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([^\s\t]*)[\s\t]*([0-9]*)/ ;
		LOG::print_msg(2,"local=$2 $3 remote=$4 $5 flags=$6") ;
		$G_slice{local_slice}[$G_slice_count] = $2 ;
		$G_slice{local_bitmap}[$G_slice_count] = $3 ;
		$G_slice{remote_slice}[$G_slice_count] = $4 ;
		$G_slice{remote_bitmap}[$G_slice_count] = $5 ;
		$G_slice{flags}[$G_slice_count] = $6 ;
		$G_slice_count++ ;
	      }
	    elsif ($command eq "master_dyn_addr")
	      {
		$MAIN::E_master_node_id = (split('\.',$2))[3] ;
	      }
	    elsif ($command eq "vice_master_dyn_addr")
	      {
		$MAIN::E_vicemaster_node_id = (split('\.',$2))[3] ;
	      }
	  }
      }
    close(FILE) ;

    open(FILE, "<$E_nhcrfs_configuration_file") ;

    while (<FILE>)
      {
	$L_line = $_ ;
	$L_line =~ s/#.*// ;
	if ($L_line =~ m/[\s\t]*([^\s\t]*)[\s\t]*=[\s\t]*([^\s\t]*)[\s\t]*/)
	  {
	    $G_nhcrfsconf{$1} = $2 ;
	  }
      }
    close(FILE) ;

    $G_config_file_loaded = TRUE ;
    return TRUE ;
    
  }

#--------------------------------------------------------------
#
#  load_partitioning
#
#--------------------------------------------------------------

sub load_partitioning
  {
    my $G_partition_cout = 0 ;
    my $started = FALSE;

    load_root_slice();

    my @result = qx{/usr/sbin/prtvtoc $G_root_slice} ;

    foreach $line (@result)
      {
	chomp($line);
	if ($line =~ m{([0-9]+) bytes/sector})
	  {
	    $G_sector = $1 ;
	    next ;
	  }
	if ($line =~ m{\* Partition})
	  {
	    $started=TRUE ;
	    next ;
	  }
	if (($started eq TRUE) && ($line =~ m/[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([^\s\t]*)/))
	  {
	    $G_partition{number}[$G_partition_count] = $1 ;
	    $G_partition{tag}[$G_partition_count] = $2 ;
	    $G_partition{flags}[$G_partition_count] = $3 ;
	    $G_partition{firstsector}[$G_partition_count] = $4 ;
	    $G_partition{countsector}[$G_partition_count] = $5 ;
	    $G_partition{lastsector}[$G_partition_count] = $6 ;
	    $G_partition{mount}[$G_partition_count] = $7 ;
	    $G_partition_count++ ;
	  }
      }
  }

#--------------------------------------------------------------
#
#  get svm partition size
#
#--------------------------------------------------------------

sub get_svm_partition_size
  {
    my ($partition) = @_;

    my @result = qx{/usr/sbin/prtvtoc $partition} ;
    my $sector = 0 ;
    my $started = FALSE ;

    foreach $line (@result)
      {
	if ($line =~ m{([0-9]+) bytes/sector})
	  {
	    $G_svm_sector = $1 ;
	    next ;
	  }
	if ($line =~ m{\* Partition})
	  {
	    $started=TRUE ;
	    next ;
	  }
	if (($started eq TRUE) && ($line =~ m/[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([0-9]+)[\s\t]*([^\s\t]*)/))
	  {
	    # return the countsector
	    return $5 ;
	  }
      }
    
    LOG::print_error("DiskSuite partition $partition doesn't exist") ;
  }

#--------------------------------------------------------------
#
#  check_partitioning
#
#--------------------------------------------------------------

sub check_partitioning
  {
    my $L_status = OK ;

    # can't be checked if SVM is used
    if ($G_use_svm eq "TRUE") {
      return ;
    }

    #
    # check that required mounting point are there and
    # that their size seems correct
    #
    foreach $mountpoint (keys %G_mountpoint)
      {
	my $found = FALSE ;
	for ($i = 0 ; $i < $G_partition_count ; $i++)
	  {
	    if ($G_partition{mount}[$i] eq $mountpoint)
	      {
		$expected = $G_mountpoint{$mountpoint}{size} / $G_sector ;
		$real = $G_partition{countsector}[$i];
		$found = TRUE ;
		if ($expected > $real)
		  {
#=item Partition size too small for mounting point <name>: <real> sectors instead of expected
#
#    Please adapt the partition size.
#
#=cut
		    $L_status = FAILED ;
		    LOG::print_error("Partition size too small for mounting point $mountpoint: $real sectors instead of $expected") ;
	      }
		last ;
	      }
	  }
	if ($found eq FALSE)
	  {

#=item No partition for the mounting point <name>
#
#    There is no partition defined for the specified mounting point.
#
#=cut
	    $L_status = FAILED ;
	    LOG::print_error("No partition for the mounting point $mountpoint") ;
	  }
      }

    LOG::print_result("Disk partitioning", $L_status) ;
  }

#--------------------------------------------------------------
#
#  check_vfstab_slice
#
#--------------------------------------------------------------

sub check_vfstab_slice
  {
    ($slice) = @_ ;

    $L_OK = TRUE ;

    #
    # check that slice is present (and not in a commented line)
    #
    chomp($result = qx{/usr/bin/grep "^[^\#]*$slice" /etc/vfstab}) ;
    if ($result eq "")
      {

#=item <slice> not defined in /etc/vfstab
#
#    All slices defined in the CRFS configuration file must have
#    an entry in /etc/vfstab.
#
#=cut

	LOG::print_error("$slice not defined in /etc/vfstab") ;
	return FALSE ;
      }

    LOG::print_msg(3, "read in /etc/vfstab: $result") ;

    #
    # check that "mount at boot" is set to "no" (6th parameter: 5th from 0)
    #
    $mount_at_boot = (split(" ", $result))[5] ;
    if ($mount_at_boot ne "no")
      {

#=item "mount at boot" for slice <slice> not set to "no"
#
#    This option prevents the slice from being automatically mounted at boot.
#    An incorrect option may cause this slice to be incorrectly
#    managed by RNFS.
#
#=cut

	LOG::print_error("\"mount at boot\" for slice $slice not set to \"no\"") ;
	$L_OK = FALSE ;
      }

    #
    # check that logging is set and forcedirectio is not set (7th parameter: 6 from 0)
    #
    $list_option = (split(" ",$result))[6] ;
    LOG::print_msg(2, "options: $list_option") ;

    $forcedirectio = FALSE ;
    $logging = FALSE ;
    foreach $option (split(",",$list_option))
      {
	LOG::print_msg(3,"    testing: $option") ;
	if ($option eq "forcedirectio")
	  {
	    $forcedirectio = TRUE ;
	    next ;
	  }
	if ($option eq "logging")
	  {
	    $logging = TRUE ;
	  }
      }

    if ($forcedirectio ne FALSE)
      {

#=item "forcedirectio" mount option set for slice <slice>
#
#    This option must not be set for the specified slice.
#
#=cut

	LOG::print_error("\"forcedirectio\" mount option set for slice $slice") ;
	$L_OK = FALSE ;
      }
    if ($logging ne TRUE)
      {

#=item "logging" mount option not set for slice <slice>
#
#    This option is required for the specified slice.
#
#=cut

	LOG::print_error("\"logging\" mount option not set for slice $slice") ;
	$L_OK = FALSE ;
      }
    return $L_OK;

  }

#--------------------------------------------------------------
#
#  check_vfstab_root
#
#--------------------------------------------------------------

sub check_vfstab_root
  {
    $L_OK = TRUE ;

    load_root_slice();

    LOG::print_msg(3, "root file system: $G_root_slice") ;

    #
    # check that slice is present (and not in a commented line)
    #
    chomp($result = qx{/usr/bin/grep "^[^\#]*$G_root_slice" /etc/vfstab}) ;
    if ($result eq "")
      {

#=item root file system <slice> not defined in /etc/vfstab
#
#    The slice corresponding to the root file system must be defined
#    in /etc/vfstab.
#
#=cut
	LOG::print_error("root file system $G_root_slice not defined in /etc/vfstab") ;
	return FALSE ;
      }

    LOG::print_msg(3, "read in /etc/vfstab: $result") ;

    #
    # check that forcedirectio is not set and logging is set (7th parameter: 6 from 0)
    #
    $list_option = (split(" ",$result))[6] ;
    LOG::print_msg(2, "options: $list_option") ;

    $forcedirectio = FALSE ;
    $logging = FALSE ;
    foreach $option (split(",",$list_option))
      {
	LOG::print_msg(3,"    testing: $option") ;
	if ($option eq "forcedirectio")
	  {
	    $forcedirectio = TRUE ;
	    next ;
	  }
	if ($option eq "logging")
	  {
	    $logging = TRUE ;
	  }
      }

    if ($forcedirectio ne FALSE)
      {

#=item "forcedirectio" mount option set for root file system <slice>
#
#    This option must not be set for the specified slice.
#
#=cut

	LOG::print_error("\"forcedirectio\" mount option set for root file system $G_root_slice") ;
	$L_OK = FALSE ;
      }
    if ($logging ne TRUE)
      {

#=item "logging" mount option not set for root file system <slice>
#
#    This option is required for the specified slice.
#
#=cut

	LOG::print_error("\"logging\" mount option not set for root file system $G_root_slice") ;
	$L_OK = FALSE ;
      }

    return $L_OK ;
  }

#--------------------------------------------------------------
#
#  check_bitmap
#
#--------------------------------------------------------------

sub check_bitmap
  {
    my $L_status = OK ;
    my $countsector ;
    my $number;
    my $bitmap_number;

    for ($i = 0 ; $i < $G_slice_count ; $i++)
      {
	# retrieve the number of the partition to be replicated
	# format looks like c0d0t0sx or is a SVM metadevice

	if ($G_slice{local_slice}[$i] =~ m{\/dev\/rdsk\/c[0-9]+t[0-9]+d[0-9]+s([0-9]+)}) {
	  $number =$1 ;
	  $countsector = $G_partition{countsector}[$number];
	} else {
	  $countsector = get_svm_partition_size($G_slice{local_slice}[$i]);
	}

	# compute the required size for bitmap slice
	# 512 bytes plus 1 bit per 32KB of replicated partition
	# count = ((size in sector * sectorsize) / (32KB)) / sectorsize + 1
	# => (size in sector / 32Kb) + 1
	my $size = $countsector * 512;
	my $bits = divrounded($size, 32*1024) ;
	my $expected_bytes = 512 + divrounded($bits, 8);

	# check the bitmap slice size
	if ($G_slice{local_bitmap}[$i] =~ m{\/dev\/rdsk\/c[0-9]+t[0-9]+d[0-9]+s([0-9]+)}) {
	  $bitmap_number =$1 ;
	  $countsector = $G_partition{countsector}[$bitmap_number];
	} else {
	  $countsector = get_svm_partition_size($G_slice{local_bitmap}[$i]);
	}
	my $bytes = $countsector * 512 ;
	LOG::print_msg(1, "slice=$G_slice{local_slice}[$i] replication bitmap size=$bytes, expected size=$expected_bytes\n") ;
	if ($bytes < $expected_bytes)
	  {

#=item Slice <slice>: replication bitmap slice <bitmap> too small: (<x> bytes instead of <y>)
#
#    The bitmap slice is too short to support the replication of the specified slice.
#    Please extend it.
#
#=cut

	    LOG::print_error("Slice $G_slice{local_slice}: replication bitmap slice $G_slice{local_bitmap}[$i] too small ($bytes bytes instead of $expected_bytes") ;
	    $L_status = FAILED ;
	  }
      }

    LOG::print_result("Bitmap slice size", $L_status) ;
  }

#--------------------------------------------------------------
#
#  check_memory_cache
#
#--------------------------------------------------------------

sub check_memory_cache
  {
    my $result ;
    my $memory_size ;
    my $cache_mem ;
    my $L_status = OK ;

    chomp($result = qx{/usr/sbin/prtconf | grep Memory}) ;
    $result =~ m/([0-9]+)/;
    $memory_size = $1;
    LOG::print_msg(1, "Memory size: $memory_size") ;

    chomp($result = qx{grep "^[^\#]*cache_mem"  $E_sd_configuration_file}) ;
    $result =~ m/cache_mem[\s\t]+[0-9]+[\s\t]+([0-9]+)/;
    $cache_mem = $1;
    LOG::print_msg(1, "Cache memory: $cache_mem") ;

    if ($memory_size eq "")
      {
	$L_status = FAILED ;
	LOG::print_error("Can not get memory size via prtconf\n") ;
      }
    elsif ($cache_mem eq "")
      {
	$L_status = FAILED ;
	LOG::print_error("Can not get cache_mem into $E_sd_configuration_file\n") ;
      }
    elsif ($memory_size < 256)
      {
	if ($cache_mem > 48)
	  {

#=item cache_mem in /etc/opt/SUNWscm/sd.cf too big: shoubd be 48 instead of <size>
#
#    On a platform with less than 256MBytes of memory, size of memory cache need to be decreased to 48.
#
#=cut

	    LOG::print_error("cache_mem in $E_sd_configuration_file too big: should be 48 instead of $cache_mem") ;
	    $L_status = FAILED ;
	  }
      }
    LOG::print_result("Cache size", $L_status) ;
  }

#--------------------------------------------------------------
#
#  check_nhcrfs
#
#--------------------------------------------------------------

sub check_nhcrfs
  {
    my $L_status = OK ;

    if ($CMM::E_variables{CMM_USE_CGTP} ne "NO") {

      if ($G_nhcrfsconf{CRFS_NICCGTP} ne "cgtp0") {
	
#=item CRFS_NICCGTP must be set to cgtp0 in <file>
#
#    cgtp0 is the only authorized value.
#
#=cut

	$L_status = FAILED ;
	LOG::print_error("CRFS_NICCGTP must be set to cgtp0 in $E_nhcrfs_configuration_file");
      }

      if ($G_nhcrfsconf{CRFS_USE_CGTP} ne "YES") {

#=item CRFS_USE_CGTP must be set to YES in <file>
#
#    If not set to YES, CGTP will not be supported.
#
#=cut

	$L_status = FAILED ;
	LOG::print_error("CRFS_USE_CGTP must be set to YES in $E_nhcrfs_configuration_file");
      }
    
      if ($G_nhcrfsconf{CRFS_NIC0} ne $CMM::E_variables{CMM_NIC0}) {

#=item CRFS_NIC0 must be set to <cmm value> in <file>
#
#    This value must be equal to the value of CMM_NIC0 in nhcmm.conf.
#
#=cut

	$L_status = FAILED ;
	LOG::print_error("CRFS_NIC0 must be set to $CMM::E_variables{CMM_NIC0} in $E_nhcrfs_configuration_file");
      }

      if ($G_nhcrfsconf{CRFS_NIC1} ne $CMM::E_variables{CMM_NIC1}) {

#=item CRFS_NIC1 must be set to <cmm value> in <file>
#
#    This value must be equal to the value of CMM_NIC1 in nhcmm.conf.
#
#=cut

	$L_status = FAILED ;
	LOG::print_error("CRFS_NIC1 must be set to $CMM::E_variables{CMM_NIC1} in $E_nhcrfs_configuration_file");
      }
    } else {

      if ($G_nhcrfsconf{CRFS_USE_CGTP} ne "NO") {

#=item CRFS_USE_CGTP must be set to NO in <file>
#
#    CGTP has been disables in the CMM configuration file.
#
#=cut

	$L_status = FAILED ;
	LOG::print_error("CRFS_USE_CGTP must be set to NO in $E_nhcrfs_configuration_file");
      }
    
      if ($G_nhcrfsconf{CRFS_NIC0} ne $CMM::E_variables{CMM_NIC0}) {

#=item CRFS_NIC0 must be set to <cmm value> in <file>
#
#    This value must be equal to the value of CMM_NIC0 in nhcmm.conf.
#
#=cut

	$L_status = FAILED ;
	LOG::print_error("CRFS_NIC0 must be set to $CMM::E_variables{CMM_NIC0} in $E_nhcrfs_configuration_file");
      }


    LOG::print_result("Option setting in $E_nhcrfs_configuration_file", $L_status) ;

    }
  }

#--------------------------------------------------------------
#
#  check_customization
#
#--------------------------------------------------------------

sub check_customization
  {
    if (OS::guess_if_master_eligible eq FALSE)
      {
	return ;
      }

    LOG::print_msg(0,"");
    LOG::print_msg(0,"CRFS checking");
    LOG::print_msg(0,"");

    if (OS::check_file("/etc/vfstab") ne TRUE)
      {
	$test = "check /etc/vfstab" ;
	LOG::print_result($test,FAILED) ;
	return FALSE ;
      } 

    $test = "check root file system definition in /etc/vfstab" ;
    $status = OK ;
    if (check_vfstab_root ne TRUE)
      {
	$status = FAILED ;
      }
    LOG::print_result($test,$status) ;
  }

#--------------------------------------------------------------
#
#  check_configuration
#
#--------------------------------------------------------------

sub check_configuration
  {
    if (CMM::is_master_eligible eq FALSE)
      {
	return ;
      }

    LOG::print_msg(0,"");
    LOG::print_msg(0,"CRFS checking");
    LOG::print_msg(0,"");

    $L_status = OK ;

    #
    # check /etc/system
    #
    if (OS::check_file("/etc/system") eq TRUE)
	{
	  chomp($result = qx{/usr/bin/grep "^[^\#]*set[ \t]*rdc:rdc_allow_force[ \t]*=[ \t]*1[ \t]*" /etc/system}) ;
	  if ($result eq "")
	    {

#=item /etc/system must contain: set rdc:rdc_allow_force = 1
#
#    The parameter rdc_allow_force must be set to the correct value.
#    A wrong value may cause RNFS to work incorrectly.
#
#=cut

	      LOG::print_error("/etc/system must contain: set rdc:rdc_allow_force = 1") ;
	      $L_status = FAILED ;
	    }
	}
    else
      {
	$L_status = FAILED ;
      }
    LOG::print_result("Option setting in /etc/system", $L_status) ;

    $L_status = OK ;
    #
    # check /usr/kernel/drv/rdc.conf
    #
    if (OS::check_file("/usr/kernel/drv/rdc.conf") eq TRUE)
      {
	chomp($result = qx{/usr/bin/grep "^[^\#]*rdc_bitmap_mode[ \t]*=[ \t]*1[ \t]*" /usr/kernel/drv/rdc.conf}) ;
	if ($result eq "")
	  {

#=item /usr/kernel/drv/rdc.conf must contain: rdc_bitmap_mode=1
#
#    Please add the setting of rdc_bitmap_mode.
#    A wrong value may cause RNFS to work incorrectly.
#
#=cut

	    LOG::print_error("/usr/kernel/drv/rdc.conf must contain rdc_bitmap_mode=1") ;
	    $L_status = FAILED ;
	  }
      }
    else
      {
	$L_status = FAILED ;
      }
    LOG::print_result("Option setting in /usr/kernel/drv/rdc.conf", $L_status) ;

    #
    # /.rhosts must containt "root"
    #
    $L_status = OK ;
    if (OS::check_file("/.rhosts") eq TRUE)
      {
	chomp($result = qx{/usr/bin/grep "^[^\#]*+[ \t]*root[ \t]*" /.rhosts}) ;
	if ($result eq "")
	  {

#=item /.rhosts must contain authorization for root "+ root"
#
#    If not set, RNFS remote synchronization will work incorrectly.
#
#=cut

	    LOG::print_error("/.rhosts must contain authorization for root \"+ root\"") ;
	    $L_status = FAILED ;
	  }
      }
    else
      {
	$L_status = FAILED ;
      }
    LOG::print_result("/.rhosts configuration", $L_status) ;

    #
    # crfs service must be defined in /etc/services
    #
    OS::check_service("crfs","tcp") ;

    #
    # all further tests are skipped if the file can't be load
    #
    if (load_config_file() ne TRUE)
      {
	LOG::print_result("CRFS configuration file access", FAILED_SKIP) ;
	return ;
      }

    #
    # check that bitmap slices are present
    #
    $G_use_svm = FALSE;

    $L_status = OK ;
    for ($i = 0 ; $i < $G_slice_count ; $i++)
      {
	LOG::print_msg(3, "check slice=$G_slice{local_slice}[$i] bitmap=$G_slice{local_bitmap}[$i]");
	if ($G_slice{local_bitmap}[$i] =~ m{\/dev\/md\/rdsk\/d[0-9]+}) {
	  $G_use_svm = TRUE ;
	} elsif ($G_slice{local_bitmap}[$i] !~ m{\/dev\/rdsk\/c[0-9]+t[0-9]+d[0-9]+s[0-9]+}) {
	  $L_status = FAILED ;
	    
#=item <slice>: <bitmap slice> does not look like a raw device
#
#    A bitmap slice must be a raw device.
#
#=cut

	  LOG::print_error("$G_slice{local_slice}[$i]: $G_slice{local_bitmap}[$i] does not look like a raw device");
	}
	elsif (!-e $G_slice{local_bitmap}[$i])
	  {
	    $L_status = FAILED ;

#=item <slice>: <bitmap slice> does not exist
#
#    A bitmap slice should exist for each slice to be replicated.
#    Please create the bitmap slice with the correct size
#    or verify its name.
#
#=cut

	    LOG::print_error("$G_slice{local_slice}[$i]: $G_slice{local_bitmap}[$i] does not exist") ;
	  }
      }
    LOG::print_result("Bitmap slices", $L_status) ;

    #
    # check that the slices are present into /etc/opt/SUNWspsv/sv.cf
    # with the cache option
    #
    $L_status = OK ;
    for ($i = 0 ; $i < $G_slice_count ; $i++)
      {
	chomp(@L_result = qx{/usr/bin/grep "^[^\#]" /etc/opt/SUNWspsv/sv.cf}) ;

	$found = FALSE ;
	foreach $L_line (@L_result)
	  {
	    $L_line =~ m/([^\s\t]*)[\s\t]*([^\s\t]*)/ ;
	    if ($1 eq $G_slice{local_slice}[$i])
	      {
		$found = TRUE ;
		if ($2 ne "cache")
		  {

#=item <slice>: cache option not set
#
#    The "cache" option must be set for the specified slice
#    in /etc/opt/SUNWspsv/sv.cf
#
#=cut

		    LOG::print_error("$G_slice{local_slice}[$i]: cache option not set") ;
		    $L_status = FALSE ;
		  }
		next ;
	      }
	  }
	if ($found eq FALSE)
	  {

#=item <slice> is not defined
#
#    The specified slice must be defined in /etc/opt/SUNWspsv/sv.cf
#    with the "cache" option.
#
#=cut

	    $L_status = FAILED ;
	    LOG::print_error("$G_slice{local_slice}[$i] is not defined") ;
	  }
      }
    LOG::print_result("Slice definition in /etc/opt/SUNWspsv/sv.cf", $L_status) ;

    #
    # check that local slice is equal to remote file for all slices
    #
    $L_status = OK ;

    for ($i = 0 ; $i < $G_slice_count ; $i++)
      {
	if ($G_slice{local_slice}[$i] ne $G_slice{remote_slice}[$i])
	  {

#=item remote slice <remote> not equal to local slice <local>
#
#    For each replicated slice specified in the CRFS configuration file,
#    the remote slice name and the local slice name must be the same.
#
#=cut

	    LOG::print_error("remote slice $G_slice{remote_slice}[$i] not equal to local slice $G_slice{local_slice}[$i] in $E_crfs_configuration_file") ;
	    $L_status = FAILED ;
	  }
      }
    LOG::print_result("Local and remote slices equality", $L_status) ;

    #
    # check that slice are correctly configured in /etc/vfstab
    #
    for ($i = 0 ; $i < $G_slice_count ; $i++)
      {
	$slice = $G_slice{local_slice}[$i] ;
	$test = "check $slice definition in /etc/vfstab" ;
	$status = OK ;
	if (check_vfstab_slice ($slice) ne TRUE)
	  {
	    $status = FAILED ;
	  }
	LOG::print_result($test,$status) ;
      }

    load_partitioning ;
    if (CMM::get_actual_role eq MASTER)
      {
	check_partitioning ;
      }
    check_bitmap ;

    check_memory_cache ;

    check_nhcrfs ;
  }

#--------------------------------------------------------------
#
#  load_rdc_set
#
#--------------------------------------------------------------

sub load_rdc_set
  {
    # RDC auto sync displayed on stderr ...
    @L_result = qx{$G_rdcadm -p 2> /dev/null} ;

    foreach $line (@L_result)
      {
	# on primary node
	if ($line =~ m/([^\s\t]*)[\s\t]*->[\s\t]*([^\s\t]*)/)
	  {
	    LOG::print_msg(2, "primary: local slice: $1, remote slice: $2\n") ;
	    $G_rdc_set{$1} = $2 ;
	  }
	else
	  {
	    # on secondary node
	    if ($line =~ m/([^\s\t]*)[\s\t]*Secondary/)
	      {
		LOG::print_msg(2, "secondary: local slice: $1\n") ;
		$G_rdc_set{$1} = "secondary" ;
	      }
	  }
      }
  }

#--------------------------------------------------------------
#
#  check_starting
#
#--------------------------------------------------------------

sub check_starting
  {

    if (CMM::is_master_eligible eq FALSE)
      {
	# at this time, no diskless checking has been developed
	return ;
      }

    LOG::print_msg(0,"");
    LOG::print_msg(0,"CRFS checking");
    LOG::print_msg(0,"");

    #
    # all further tests are skipped if the file can't be load
    #
    if (load_config_file() ne TRUE)
      {
	LOG::print_result("CRFS configuration file access", FAILED_SKIP) ;
	return ;
      }

    load_rdc_set ;

    $L_status = OK ;
    if (CMM::is_master_eligible eq TRUE)
      {
	#
	# check that the slices are replicated
	#
	for ($i = 0; $i < $G_slice_count ; $i++)
	  {
	    $L_status = OK ;

	    $slice = $G_slice{local_slice}[$i] ;
	    LOG::print_msg(2, "ckeck replication of $slice") ;

	    if (! defined($G_rdc_set{$slice}))
	      {

#=item <slice> is not replicated
#
#    The specified slice, defined in the CRFS configuration slice,
#    is incorrectly replicated.
#
#=cut

		LOG::print_error("$slice is not replicated") ;
		$L_status = FAILED ;
	      }
	    LOG::print_result("slice $slice replicated", $L_status) ;
	  }
      }
  }
#--------------------------------------------------------------
#
#  display 
#
#--------------------------------------------------------------

sub display
  {
    if (CMM::is_master_eligible eq FALSE)
      {
	return ;
      }

    # we used the standard command: outputs are not re-formatted

    print "\n" ;
    print "CRFS configuration information\n" ;
    print "\n" ;

    # display volume sets information
    print "Outputs of rdcadm -p\n" ;
    print "\n" ;
    $result = system("$G_rdcadm -p") ;
    if ($result ne 0)
      {
	print "    Error when executing rdcadm\n" ;
	return ;
      }

  }

#--------------------------------------------------------------
#
#  main 
#
#--------------------------------------------------------------

#=back

#=cut

{ 
} 
