#************************************************************ 
#
# Component      CNF  
#
# Synopsis       Operating System and Packages
#
#
# Copyright 2001 Sun Microsystems, Inc. All rights reserved.
#
#
#************************************************************
#
# #ident "@(#)OS.pm 1.15 02/03/15 SMI"
#
#************************************************************

#=head1 Operating System and Packages
#
#    This section also contains descriptions for error than can be detected
#    in various circumstances (generic checking).
#
#=over 4
#
#=cut

package OS ;

use LOG ;
use APP ;

my $G_category = "OS" ;

my $G_OSname ;
my $G_OSrelease ;

#
#  constant to be checker
#
my $G_expected_OSrelease = "5.8" ;
my $G_expected_OSversion = "Generic_108528-13" ;

#
#  Packages list
#
#  1: name of the package
#  2: local (LOCAL), shared (SHARED)
#  3: version or list of versions (when EXACT is tested) separated by a comma
#  4: on master-eligible (YES or NO)
#  5: on diskless (YES or NO)
#  6: check exactly one of the listed version (EXACT) or at least the specific version (AT_LEAST)
#

my $G_package_name     = 0 ;
my $G_package_location = 1 ;
my $G_package_version  = 2 ;
my $G_package_men      = 3 ;
my $G_package_nmen     = 4 ;
my $G_package_comp     = 5 ;

my @G_packages = (
		  # ADM packages
		  ["SUNWnhadm", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,

		  # CGTP packages
		  ["SUNWnhtp8", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhtu8", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,

		  # CMM packages
		  ["SUNWnhcdt", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhcmd", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhcma", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhcmb", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,

		  # RNFS packages
		  ["SUNWspuni", "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWscm",   "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWspsv",  "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWrdcr",  "LOCAL",       "2.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWrdcu",  "LOCAL",       "2.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhfsa", "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhfsb", "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,

		  # PMD packages
		  ["SUNWnhpma", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhpmb", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhpms", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,

		  # JDK packages
		  ["SUNWj3rt",  "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,

		  # MA and JDMK packages
		  ["SUNWnhmas", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNW2jdrt", "SHARED",      "4.2",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhmaj", "SHARED",      "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhmal", "SHARED",      "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhmad", "SHARED",      "1.0",    "YES", "NO",  AT_LEAST] ,

		  # RBS packages
		  ["SUNWnhrbb", "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhrbs", "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		 ) ;

my @G_app_packages = (
		  # Application Services packages
		  ["SUNWnhapp", "LOCAL",       "1.0",    "YES", "YES", AT_LEAST] ,
		  ["SUNWnhhad", "SHARED",      "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhccs", "SHARED",      "1.0",    "YES", "NO",  AT_LEAST] ,
		  ["SUNWnhmes", "SHARED",      "1.0",    "YES", "NO",  AT_LEAST] ,

		  # LDAP packages
		  ["SUNWnhlds", "LOCAL",       "1.0",    "YES", "NO",  AT_LEAST] ,
		 ) ;


#
#  Patch list
#
#  1: name of the patch
#  2: local (LOCAL), shared (SHARED)
#  3: on master-eligible (YES or NO)
#  4: on diskless (YES or NO)

my $G_patch_name     = 0 ;
my $G_patch_location = 1 ;
my $G_patch_men      = 2 ;
my $G_patch_nmen     = 3 ;

my @G_patches = (
		  # CGTP patch
		  ["112281-01", "LOCAL",  "YES", "YES" ] ,
		  # JDMK patch
		  ["112045-02", "SHARED", "YES", "YES" ] ,
		 ) ;

#
# package acessibility
#
my $G_swdb_local_dir = "/SUNWcgha/local/export/services" ;
my $G_swdb_services = "/SUNWcgha/services" ;
my $G_swdb_mounting_point = "/SUNWcgha/swdb" ;
my $G_swdb_accessible = FALSE ;
my $G_swdb_remote_access = FALSE ;  
my $G_swdb_dir = "" ;
my $G_swdb_basedir = "" ;
my $G_swdb_path_confshare = "var/sadm/system/admin" ;
my $G_swdb_file_confshare = "INST_RELEASE" ;

#--------------------------------------------------------------
#
#  determine_swdb
#
#  shared package info database is
#  . only accessible from one node during installation
#    (when mounting points are not yet defined)
#  . accessible from all noeds after reboot (mounting point
#    has been defined on all nodes)
#
#--------------------------------------------------------------

sub determine_swdb
  {
    # is the mounting point defined ?
    $result = qx{/usr/sbin/mount | /usr/bin/grep $G_swdb_mounting_point} ;

    if ($result eq "") {
      # no mounting point: check if we are on the main node for installation
      if ( -d $G_swdb_local_dir ) {
	$G_swdb_accessible = TRUE ;
	$G_swdb_dir = $G_swdb_local_dir ;
      }

    } else {

      #
      # determine the base dir
      #
      $result = qx{/usr/sbin/mount | /usr/bin/grep $G_swdb_services};
      if ($result eq "") {
	LOG::print_error("Can't determine current BASE_DIR for shared packages") ;
	return ;
      }
      # extract the remote sub-directory name as the basedir
      if ($result !~ m{$G_swdb_local_dir/([^\s\t]*)}) {
 	LOG::print_error("Can't determine current BASE_DIR for shared packages") ;
	return ;
      }
      $G_swdb_basedir = "/$1";
      $G_swdb_accessible = TRUE ;
      $G_swdb_remote_access = TRUE ;
      $G_swdb_dir = $G_swdb_mounting_point ;
    }
	
  }

#--------------------------------------------------------------
#
#  guess_if_master_eligible
#
#    try to guess the node role by testing the mounting point
#    of the root partition since CMM can't be used
#
#--------------------------------------------------------------

sub guess_if_master_eligible
  {
    chomp($result = qx{mount -v | /usr/bin/grep " on / type "}) ;
    $result =~ m/ on \/ type ([a-z]*) /;
    if ($1 eq "ufs")
      {
	return TRUE ;
      }

    return FALSE ;
  }

#--------------------------------------------------------------
#
#  compare_version
#
#--------------------------------------------------------------

sub compare_version {

  my ($actual, $expected, $option) = @_ ;
 
  if ($option eq EXACT)
      {
	foreach $version (split(",",$expected))
	  {
	    if ($actual eq $version)
	      {
		return TRUE;
	      }
	  }
	return FALSE ;
      }
    else
      {
	if ($L_comp eq AT_LEAST)
	  {
	    @part_current = split('\.', $actual) ;
	    @part_expected = split('\.', $expected) ;
	    $atleast = TRUE ;
	    $end = FALSE ;
	    $exp_size = scalar(@part_expected) ;
	    $cur_size = scalar(@part_current) ;
	    my $i = 0 ;
	    while (($i < $exp_size) and ($end eq FALSE))
	      {
		if ($i < $cur_size)
		  {
		    if ($part_current[$i] > $part_expected[$i])
		      {
			$end = TRUE ;
		      }
		    else
		      {
			if ($part_current[$i] < $part_expected[$i])
			  {
			    $atleast = FALSE ;
			    $end = TRUE ;
			  }
		      }
                  }
		else
		  {
		    $atleast = FALSE ;
		    $end = TRUE ;
		  }
		$i++ ;
	      }
	      
	    if ($atleast eq FALSE)
	      {
		return LESS ;
	      }
	  }
      }
}

#--------------------------------------------------------------
#
#  get_package
#
#    get info about a specified package
#
#    1 : in: name of the package
#    2 : in: location
#
#    3 : out: version of the package
#    4 : out: complete name
#    5 : out: base directory
#
#    return :
#    . OK if the package exists
#    . NA if the package is not part of this node (Not Applicable)
#    . NOK if the package doesn't exist
#
#--------------------------------------------------------------

sub get_package
  {
    ($L_package, $L_location, $R_version, $R_name, $R_basedir) = @_ ;

    my $command = "" ;
    my $dir = "" ;

    # basedir may be not present in package information so it should be initialized
    $$R_basedir = "" ; 

    if ($L_location eq "SHARED") {
      $dir = "$G_swdb_dir" ;
      $command = "-R $G_swdb_dir" ;
    } elsif ($L_location ne "LOCAL") {
      LOG::print_msg("Invalid location ($L_location) for package $L_package") ;
      exit 1;
    }

    $L_result = system("/usr/bin/pkginfo $command $L_package > /dev/null 2>&1");

    if ($L_result != 0)
      {
	return NOK ;
      }

    @L_output = qx{/usr/bin/pkginfo $command -l $L_package} ;
 
    foreach $L_line (@L_output)
      {
	chomp($L_line) ;
	if ($L_line =~ m/ERROR:/)
	  {
	    return NOK ;
	  }
	if ($L_line =~ m/VERSION:[\s\t]*([0-9.]*)/)
	  {
	    $$R_version = $1 ;
	  }
	if ($L_line =~ m/NAME:[\s\t]*(.*)/)
	  {
	    $$R_name = $1 ;
	  }
	if ($L_line =~ m/BASEDIR:[\s\t]*(.*)/)
	  {
	    $$R_basedir = $1 ;

	    # for a shared package:
	    # the base dir is checked only if the package is accessible
	    # via the mounting point (meaning that the base dir has been retrieved)
	    # -> if the base dir is different, the package is not the right one
	    #    we consider it as not installed
	    #
	    # if the access is local (before mounting point has been set)
	    # the base dir was not retrieved so don't check anything !
	    #
	    #
	    if (($L_location eq "SHARED") 
		&& ($G_swdb_remote_access eq TRUE) 
		&& ($$R_basedir ne $G_swdb_basedir)) {
	      return NOK ;
	    }
	  }
      }

    return OK ;
      
  }

#--------------------------------------------------------------
#
#  check_package
#
#    check if a package is installed and if its version
#    is correct
#
#    1 : name of the package
#    2 : location
#    3 : version to be tested (can be a list if type=EXACT
#    4 : type of comparison
#        EXACT   : must match the exact version
#        AT_LEAST: minimal version
#
#--------------------------------------------------------------

sub check_package
  {
    ($L_package, $L_location, $L_version, $L_comp) = @_ ;

    my $L_test = "Package $L_package" ;
    my $L_status = OK ;
    my $L_current_version ;
    my $L_current_name ;
    my $L_basedir ;

    my $return = get_package($L_package, $L_location, 
			     \$L_current_version, \$L_current_name, \$L_basedir) ;
#    if ($return eq NA)
#      {
#	return ;
#      }

    if ($return ne OK)
      {

#=item package <name> not installed.
#
#    Please install the specified package.
#
#=cut

	LOG::print_error("package $L_package not installed") ;
	LOG::print_result($L_test,FAILED) ;
	return ;
      }

    $status = compare_version($L_current_version, $L_version, $L_comp) ;

    if ($status eq FALSE)
      {

#=item package <name>: version <current> instead of <expected>
#
#    Please install the correct version of the specified package.
#
#=cut

	LOG::print_error("package $L_package: version $L_current_version instead of $L_version") ;
	$L_status = FAILED ;
      }
    else
      {
	if ($status eq "LESS")
	  {

#=item package <name>: version <current> (at least <minimum> required)
#
#    Please install a correct version of the specified package.
#
#=cut

	    LOG::print_error("package $L_package: version $L_current_version (at least $L_version required)") ;
	    $L_status = FAILED ;
	  }
      }
    
    LOG::print_result($L_test,$L_status) ;
  }


#--------------------------------------------------------------
#
#  check_package_array
#
#    check if a array of package is installed
#
#    1: package array
#    2: MEN or NMEN
#
#--------------------------------------------------------------

sub check_package_array
  {
    my ($pkgarr, $role) = @_ ;
    
    my $field ;
    
    if ($role eq "MEN") {
      $field = $G_package_men ;
    } else {
      $field = $G_package_nmen ;
    }
    
    #
    # check local packages if it makes sense
    #

    LOG::print_msg(0,"") ;
    LOG::print_msg(0,"Local packages") ;
    LOG::print_msg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if ((@$pkg[$field] eq "YES") && (@$pkg[$G_package_location] eq "LOCAL"))
	  {
	    check_package(@$pkg[$G_package_name], 
			  @$pkg[$G_package_location], 
			  @$pkg[$G_package_version],
			  @$pkg[$G_package_comp]);
	  }
      }

    if ($G_swdb_accessible eq FALSE) {
      LOG::print_msg(0,"") ;
      LOG::print_msg(0,"Shared packages currently not accessible") ;
      LOG::print_msg(0,"") ;
      return ;
    }

    #
    # check shared packages
    #
    LOG::print_msg(0,"") ;
    LOG::print_msg(0,"Shared packages (on $G_swdb_dir)") ;
    LOG::print_msg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if (@$pkg[$G_package_location] eq "SHARED")
	  {
	    check_package(@$pkg[$G_package_name], 
			  @$pkg[$G_package_location], 
			  @$pkg[$G_package_version],
			  @$pkg[$G_package_comp]);
	  }
      }
  }

#--------------------------------------------------------------
#
#  get_patch
#
#    get info about a specified patch
#
#    1 : in: name of the package
#    2 : in: location
#
#    return TRUE if the patch exists
#
#--------------------------------------------------------------

sub get_patch
  {
    ($L_patch, $L_location) = @_ ;

    my $optcmd = "" ;

    if ($L_location eq "SHARED") {
      $optcmd = "-R $G_swdb_dir" ;
    } elsif ($L_location ne "LOCAL") {
      LOG::print_msg("Invalid location ($L_location) for patch $L_patch") ;
      exit 1;
    }

    $L_result = qx{/usr/sbin/patchadd $optcmd -p | grep $L_patch};

    if ($L_result eq "")
      {
	return FALSE ;
      }

    return TRUE ;
  }

#--------------------------------------------------------------
#
#  check_patch
#
#    check if a patch is installed and if its version
#    is correct
#
#    1 : name of the patch
#    2 : location
#
#--------------------------------------------------------------

sub check_patch
  {
    ($L_patch, $L_location) = @_ ;

    my $L_test = "Patch $L_patch" ;
    my $L_status = OK ;

    if (($L_location eq "SHARED") && ($G_swdb_accessible eq FALSE)) {
      return ;
    }

    if (get_patch($L_patch, $L_location) ne TRUE)
      {

#=item Patch <patch> not installed
#
#    The specified patch must be installed.
#
#=cut

	LOG::print_error("Patch $L_patch not installed") ;
	$L_status = FAILED;
      }
    LOG::print_result($L_test,$L_status) ;
  }

#--------------------------------------------------------------
#
#  check_file
#
#    1 : name of the file to check (exists, readable)
#
#--------------------------------------------------------------

sub check_file
  {
    ($filename) = @_ ;

    if (! -e $filename)
      {

#=item <file> does not exist
#
#    This may be caused by a wrong pathname.
#
#=cut

	LOG::print_error("$filename does not exist") ;
	return FALSE ;
     }

    if (! -r $filename)
      {

#=item can not read <file>: permission denied
#
#    Please check that the root account can access the specified file.
#
#=cut

        LOG::print_error("can't read $filename: permission denied") ;
	return FALSE ;
      }

    return TRUE ;
  }

#--------------------------------------------------------------
#
#  check_service
#
#    check if a service is defined in /etc/services
#
#    1 : name of the service
#    2 : protocol (tcp/udp)
#
#--------------------------------------------------------------

sub check_service
  {
    ($L_service,$L_protocol) = @_ ;

    my $L_status = OK ;

    LOG::print_msg(2,"Check service $L_service, protocol $L_protocol") ;

    #
    # crfs service must be defined in /etc/services
    # a service can appear twice with different protocol
    #
    $L_status = OK ;
    if (check_file("/etc/services") eq TRUE)
      {
	open(FH,"</etc/services") or die "/etc/services: $!";
	$found = FALSE ;
	$wrong_protocol = FALSE ;
	while (<FH>)
	  {
	    chomp($line=$_) ;
	    $line =~ s/#.*// ;
	    if ($line =~ m/[\s\t]*([^\s\t]*)[\s\t]*([0-9]*)\/([a-z]*)/)
              {
	        $service = $1 ;
	        $port = $2 ;
	        $protocol = $3 ;
	        if ($service eq $L_service)
	          {
		    if ($protocol eq $L_protocol)
		      {
		        $found = TRUE ;
		        $wrong_protocol = FALSE ;
		        next ;
		      }
		    else
		      {
		        $wrong_protocol = TRUE ;
		      }
	          }
	      }
           }

	if ($found eq TRUE)
	  {
	    $L_STATUS = OK ;
	    LOG::print_result("$L_service defined in /etc/services", $L_status) ;
	  }
	else
	  {
	  if ($wrong_protocol eq TRUE)
	    {

#=item <service> service: <current> instead of <expected>
#
#    Please correct the protocol for the specified service
#    in /etc/services.
#
#=cut

	      LOG::print_error ("$L_service service: $protocol instead of $L_protocol");
	      $L_status = FAILED ;
	    }
	  else
	    {

#=item <service> service must be defined in /etc/services
#
#    Please add an entry for the specified service in /etc/services.
#    This may cause daemons using it to work incorrectly.
#
#=cut
 
	      LOG::print_error("$L_service service must be defined in /etc/services") ;
	      $L_status = FAILED ;
	    }
	}
      }

    close (FH) ;
  }

#--------------------------------------------------------------
#
#  check_installation
#
#      check information related to the Solaris OS
#
#--------------------------------------------------------------

sub check_installation
  {
    my $L_status = OK ;

    LOG::print_msg(0, "") ;
    LOG::print_msg(0, "OS and Software checking") ;
    LOG::print_msg(0, "") ;

    $L_test = "64-bit kernel mode" ;
    my $bits = qx{/usr/bin/isainfo -b} ;
    if ($bits != "64") {
#=item Invalid kernel mode: <bits>-bit instead of 64-bit mode
#
#    Please install the correct Solaris version.
#
#=cut

	LOG::print_error("Invalid kernel mode: $bits-bit instead of 64-bit mode") ;
	$L_status = FAILED;
      }
    LOG::print_result($L_test,$L_status) ;

    $L_status = OK ;
    chomp($G_OSname    = qx{/usr/bin/uname -s}) ;
    chomp($G_OSrelease = qx{/usr/bin/uname -r}) ;
    chomp($G_OSversion = qx{/usr/bin/uname -v}) ;

    LOG::print_msg(3, "Current release: $G_OSrelease, expected release: $G_expected_OSrelease") ;
    $L_test = "OS release" ;
    if ($G_OSrelease ne $G_expected_OSrelease)
      {

#=item release <current> instead of <expected>
#
#    Please install the correct Solaris version.
#
#=cut

	LOG::print_error("release $G_OSrelease instead of $G_expected_OSrelease") ;
	$L_status = FAILED_SKIP;
	LOG::print_result($L_test,$L_status) ;
      }
    else
      {
	LOG::print_result($L_test,$L_status) ;

	LOG::print_msg(3,"Version $G_OSversion, expected $G_expected_OSversion") ;
	$L_test = "OS version" ;
	if ($G_OSversion ne $G_expected_OSversion)
	  {

#=item version <current> instead of <expected>
#
#    Please install the correct Solaris version.
#
#=cut

	    LOG::print_error("version $G_OSversion instead of $G_expected_OSversion") ;
	    $L_status = FAILED;
	  }
	LOG::print_result($L_test,$L_status) ;
      }

    #
    #  check various packages
    #

    determine_swdb ;

    if (guess_if_master_eligible eq TRUE)
      {
	check_package_array(\@G_packages, "MEN") ;
	if (APP::app_installed eq TRUE)
	  {
	    check_package_array(\@G_app_packages, "MEN") ;
	  }
      }
    else
      {
	check_package_array(\@G_packages, "NMEN") ;
	if (APP::app_installed eq TRUE)
	  {
	    check_package_array(\@G_app_packages, "NMEN") ;
	  }
      }

    #
    #  check various patches
    #
    LOG::print_msg(0, "") ;
    LOG::print_msg(0, "Patches") ;
    LOG::print_msg(0, "") ;
    for ($i = 0 ; $i < scalar(@G_patches) ; $i++)
      {
	check_patch($G_patches[$i][0], 
		    $G_patches[$i][1]) ;
      }

    #
    # check if shared file for patchabitlity support is present 
    # 
    if ($G_swdb_accessible eq TRUE) {
      $L_status = FAILED ;
      if (-f "$G_swdb_dir/$G_swdb_path_confshare/$G_swdb_file_confshare") {
	$L_status = OK ;
      } else {
	
#=File for patchability of shared packages not present.
#
#    Please use the "nhadm confshare" command to create this file.
#
#=cut

	LOG::print_error("File for patchability of shared packages not present.") ;
      }
    LOG::print_result("INST_RELEASE file for shared packages", $L_status) ;
    }
  }

#--------------------------------------------------------------
#
#  display_package_array
# 
#      display information regarding package array
#
#--------------------------------------------------------------

sub display_package_array
  {
    my ($pkgarr, $role) = @_ ;

    my $field ;
    my $L_name;
    my $L_location ;
    my $L_version ;
    my $L_basedir ;

    if ($role eq "MEN") {
      $field = $G_package_men ;
    } else {
      $field = $G_package_nmen ;
    }

    determine_swdb ;

    #
    # display local packages
    #
    LOG::print_msg(0,"") ;
    LOG::print_msg(0,"Local packages") ;
    LOG::print_msg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if ((@$pkg[$field] eq "YES") && (@$pkg[$G_package_location] eq "LOCAL"))
         {
	   $L_name = @$pkg[$G_package_name];
	   $L_location = @$pkg[$G_package_location];
	   my $return = get_package($L_name, $L_location, \$L_version, \$L_name, \$L_basedir) ;
	   if ($return eq OK)
	     {
	       print  "    package $L_package\n" ;
	       print  "        name:        $L_name\n" ;
	       print  "        version:     $L_version\n" ;
	       if ($L_basedir ne "") {
		 print  "        basedir:     $L_basedir\n" ;
	       }
	     }
	   elsif ($return eq NOK)
	     {
	       printf "    package %-12s -> not installed\n", $L_package ;
	     }
	}
      }

    if ($G_swdb_accessible eq FALSE) {
      LOG::print_msg(0,"") ;
      LOG::print_msg(0,"Shared packages currently not accessible") ;
      LOG::print_msg(0,"") ;
      return ;
    }

    #
    # display shared packages
    #
    LOG::print_msg(0,"") ;
    LOG::print_msg(0,"Shared packages (on $G_swdb_dir)") ;
    LOG::print_msg(0,"") ;
    foreach $pkg (@$pkgarr)
      {
	if (@$pkg[$G_package_location] eq "SHARED")
         {
	   $L_name = @$pkg[$G_package_name];
	   $L_location = @$pkg[$G_package_location];
	   my $return = get_package($L_name, $L_location, \$L_version, \$L_name, \$L_basedir) ;
	   if ($return eq OK)
	     {
	       print  "    package $L_package\n" ;
	       print  "        name:        $L_name\n" ;
	       print  "        version:     $L_version\n" ;
	       if ($L_basedir ne "") {
		 print  "        basedir:     $L_basedir\n" ;
	       }
	     }
	   elsif ($return eq NOK)
	     {
	       printf "    package %-12s -> not installed\n", $L_package ;
	     }
	}
      }
  }
#--------------------------------------------------------------
#
#  display
#
#      display information regarding OS
#
#--------------------------------------------------------------

sub display
  {
    print "\n" ;
    print "System information\n" ;
    print "\n" ;

    chomp($G_hardware  = qx{/usr/bin/uname -i}) ;
    chomp($G_OSname    = qx{/usr/bin/uname -s}) ;
    chomp($G_OSrelease = qx{/usr/bin/uname -r}) ;
    chomp($G_OSversion = qx{/usr/bin/uname -v}) ;

    print "    node name:  $MAIN::E_node_name\n" ;
    print "    Hardware:   $G_hardware\n" ;
    print "    OS release: $G_OSrelease\n" ;
    print "    OS version: $G_OSversion\n" ;

    if (guess_if_master_eligible eq TRUE)
      {
	display_package_array(\@G_packages, "MEN") ;
	if (APP::app_installed eq TRUE)
	  {
	    display_package_array(\@G_app_packages, "MEN") ;
	  }
      } 
    else
      {
	display_package_array(\@G_packages, "NMEN") ;
	if (APP::app_installed eq TRUE)
	  {
	    display_package_array(\@G_app_packages, "NMEN") ;
	  }
      }
  }

#--------------------------------------------------------------
#
#  configure_shared
#
#--------------------------------------------------------------

sub configure_shared
  {
    my ($dir) = @_ ;

    if ($dir eq "") {
      #
      # the default directory is used if accessible
      #
      determine_swdb ;
      if ($G_swdb_accessible eq FALSE) {
	LOG::print_error("Directory for shared packages not accessible") ;
	return ;
      }
      $dir = $G_swdb_dir ;
    }

    $result = qx{/usr/bin/mkdir -p $dir/$G_swdb_path_confshare 2>&1} ;
    if ( $? ne 0 ) {
      LOG::print_msg(0,"Error when executing mkdir") ;
      LOG::print_msg(0,$result) ;
      exit 1 ;
    }
    $result = qx{/usr/bin/cp /$G_swdb_path_confshare/$G_swdb_file_confshare $dir/$G_swdb_path_confshare/$G_swdb_file_confshare 2>&1} ;
    if ( $? ne 0 ) {
      LOG::print_msg(0,"Error when executing copy") ;
      LOG::print_msg(0,$result) ;
      exit 1 ;
    }

    LOG::print_msg(0, "File $dir/$G_swdb_path_confshare/$G_swdb_file_confshare created") ;
  }

#--------------------------------------------------------------
#
#  main
#
#--------------------------------------------------------------

#=back
#
#=cut

{
}
